use super::{
    ConcreteDatum, ConcreteDatumList, Datum, DatumList, DatumVisitor, QueryEquivalent,
    QueryEquivalentList, StringQuery,
};

impl DatumList for () {
    const IS_EMPTY: bool = true;
    // arbitrary Datum, just as a placeholder
    type ListHead = bool;
    type ListTail = ();

    type RefList<'l> = () where Self: 'l;

    fn list_head(&self) -> &Self::ListHead {
        unreachable!()
    }
    fn list_tail(&self) -> Self::ListTail {}

    fn accept(&self, _: &mut impl DatumVisitor) {}
}
impl ConcreteDatumList for () {
    fn build_equivalent<'l>(
        _from: impl Iterator<Item = &'l str>,
    ) -> Option<impl QueryEquivalentList<Self> + 'l> {
        Some(())
    }
}

impl QueryEquivalentList<()> for () {}

impl<T: Datum> DatumList for T {
    type ListHead = T;
    type ListTail = ();
    type RefList<'l> = () where Self: 'l;

    fn list_head(&self) -> &Self::ListHead {
        self
    }
    fn list_tail(&self) -> Self::ListTail {}

    fn accept(&self, visitor: &mut impl DatumVisitor) {
        visitor.visit(self);
    }
}

impl<T: ConcreteDatum> ConcreteDatumList for T {
    fn build_equivalent<'l>(
        mut from: impl Iterator<Item = &'l str>,
    ) -> Option<impl QueryEquivalentList<Self> + 'l> {
        Some(StringQuery(from.next()?))
    }
}

impl<T: ConcreteDatum, E: QueryEquivalent<T>> QueryEquivalentList<T> for E {}

impl<T0: Datum> DatumList for (T0,) {
    type ListHead = T0;
    type ListTail = ();
    type RefList<'l> = (&'l T0,) where Self: 'l;

    fn list_head(&self) -> &Self::ListHead {
        &self.0
    }
    fn list_tail(&self) -> Self::ListTail {}

    fn accept(&self, visitor: &mut impl DatumVisitor) {
        visitor.visit(&self.0);
    }
}

impl<T0: ConcreteDatum> ConcreteDatumList for (T0,) {
    fn build_equivalent<'l>(
        mut from: impl Iterator<Item = &'l str>,
    ) -> Option<impl QueryEquivalentList<Self> + 'l> {
        Some((StringQuery(from.next()?),))
    }
}

impl<T0: ConcreteDatum, E0: QueryEquivalent<T0>> QueryEquivalentList<(T0,)> for (E0,) {}

macro_rules! datum_list {
    ($len:literal, $ty0:ident : $e0 : ident : $n0: tt, $($ty:ident : $e:ident : $n:tt),+) => {
        impl<$ty0: Datum, $($ty: Datum),*> DatumList for ($ty0, $($ty),*) {
            type ListHead = $ty0;
            type ListTail = ( $( $ty ),* , );
            type RefList<'l> = ( &'l $ty0, $( &'l $ty ),* ) where Self: 'l;

            fn list_head(&self) -> &Self::ListHead { &self.0 }
            fn list_tail(&self) -> Self::ListTail {
                todo!()
            }

            fn accept(&self, visitor: &mut impl DatumVisitor) {
                visitor.visit(&self. $n0);
                $(visitor.visit(&self. $n));*
            }
        }

        impl<$ty0: ConcreteDatum, $e0: QueryEquivalent<$ty0>, $( $ty: ConcreteDatum, $e: QueryEquivalent<$ty> ),*> QueryEquivalentList<( $ty0, $( $ty ),* )> for ( $e0, $( $e ),* ) {}

        impl<$ty0: ConcreteDatum, $( $ty: ConcreteDatum ),*> ConcreteDatumList for ($ty0, $($ty),*) {
            fn build_equivalent<'l>(mut from: impl Iterator<Item = &'l str>) -> Option<impl QueryEquivalentList<Self> + 'l> {
                #[allow(clippy::eq_op)]
                Some((
                    StringQuery( from.next()? ),
                    $( if $n == $n { StringQuery( from.next()? ) } else { panic!() } ),*
                ))
            }
        }
    }
}

datum_list!(2, T0:E0:0, T1:E1:1);
datum_list!(3, T0:E0:0, T1:E1:1, T2:E2:2);
datum_list!(4, T0:E0:0, T1:E1:1, T2:E2:2, T3:E3:3);
datum_list!(5, T0:E0:0, T1:E1:1, T2:E2:2, T3:E3:3, T4:E4:4);
datum_list!(6, T0:E0:0, T1:E1:1, T2:E2:2, T3:E3:3, T4:E4:4, T5:E5:5);
datum_list!(7, T0:E0:0, T1:E1:1, T2:E2:2, T3:E3:3, T4:E4:4, T5:E5:5, T6:E6:6);
datum_list!(8, T0:E0:0, T1:E1:1, T2:E2:2, T3:E3:3, T4:E4:4, T5:E5:5, T6:E6:6, T7:E7:7);
datum_list!(9, T0:E0:0, T1:E1:1, T2:E2:2, T3:E3:3, T4:E4:4, T5:E5:5, T6:E6:6, T7:E7:7, T8:E8:8);
datum_list!(10, T0:E0:0, T1:E1:1, T2:E2:2, T3:E3:3, T4:E4:4, T5:E5:5, T6:E6:6, T7:E7:7, T8:E8:8, T9:E9:9);
datum_list!(11, T0:E0:0, T1:E1:1, T2:E2:2, T3:E3:3, T4:E4:4, T5:E5:5, T6:E6:6, T7:E7:7, T8:E8:8, T9:E9:9, T10:E10:10);
datum_list!(12, T0:E0:0, T1:E1:1, T2:E2:2, T3:E3:3, T4:E4:4, T5:E5:5, T6:E6:6, T7:E7:7, T8:E8:8, T9:E9:9, T10:E10:10, T11:E11:11);
datum_list!(13, T0:E0:0, T1:E1:1, T2:E2:2, T3:E3:3, T4:E4:4, T5:E5:5, T6:E6:6, T7:E7:7, T8:E8:8, T9:E9:9, T10:E10:10, T11:E11:11, T12:E12:12);
datum_list!(14, T0:E0:0, T1:E1:1, T2:E2:2, T3:E3:3, T4:E4:4, T5:E5:5, T6:E6:6, T7:E7:7, T8:E8:8, T9:E9:9, T10:E10:10, T11:E11:11, T12:E12:12, T13:E13:13);
datum_list!(15, T0:E0:0, T1:E1:1, T2:E2:2, T3:E3:3, T4:E4:4, T5:E5:5, T6:E6:6, T7:E7:7, T8:E8:8, T9:E9:9, T10:E10:10, T11:E11:11, T12:E12:12, T13:E13:13, T14:E14:14);
datum_list!(16, T0:E0:0, T1:E1:1, T2:E2:2, T3:E3:3, T4:E4:4, T5:E5:5, T6:E6:6, T7:E7:7, T8:E8:8, T9:E9:9, T10:E10:10, T11:E11:11, T12:E12:12, T13:E13:13, T14:E14:14, T15:E15:15);
