// this is overcomplicated.
// just give a function that has access to each entity type and allow it to copy whatever it likes?

#![allow(missing_docs)]

use std::collections::HashSet;

use crate::{
    db::ConnectionLease,
    query::Queryable,
    schema::Schema,
    DBResult,
};

mod new;

pub use new::New;
use new::WrapSchema;

use super::{
    build::{collect_from_database, DatabaseSchema},
    meta::MetadataDB,
};

pub trait Migration: 'static + Default {
    type OldSchema: Schema + Default;
    type NewSchema: Schema + Default;
}

pub trait MigrationRef {
    // fn with_new_schema(&self, changed: HashSet<String>) -> Box<&dyn Schema>;
    fn collect_intermediate_schema(&self, towrap: HashSet<String>) -> Box<&dyn Schema>;
}
impl<M: Migration> MigrationRef for M {
    fn collect_intermediate_schema(&self, towrap: HashSet<String>) -> Box<&dyn Schema> {
        let wrap_schema = WrapSchema::<M::NewSchema>::new(M::NewSchema::default(), towrap);

        println!("collection: {:?}", collect_from_database(&wrap_schema));
        todo!()
    }
}

struct MigrationEntry {
    old_schema: DatabaseSchema,
    old_signature: String,
    new_schema: DatabaseSchema,
    new_signature: String,
    migration: Box<dyn MigrationRef>,
}

#[derive(microrm_macros::Schema, Default)]
struct EmptySchema {}

pub struct Migrator {
    base_schema: DatabaseSchema,
    migrations: Vec<MigrationEntry>,
}

impl Migrator {
    pub fn new<Base: Schema + Default>() -> Self {
        Self {
            base_schema: collect_from_database(&Base::default()),
            migrations: vec![],
        }
    }

    pub fn add<M: Migration>(&mut self) {
        let old_schema = collect_from_database(&M::OldSchema::default());
        let new_schema = collect_from_database(&M::NewSchema::default());
        self.migrations.push(MigrationEntry {
            old_signature: old_schema.signature().into(),
            new_signature: new_schema.signature().into(),
            old_schema,
            new_schema,
            migration: Box::new(M::default()),
        });
    }

    pub fn apply(&self, lease: &mut ConnectionLease) -> DBResult<()> {
        log::debug!("Applying migrations");
        // try getting data from the metaschema
        let existing_schema = MetadataDB::default()
            .metastore
            .keyed(DatabaseSchema::SCHEMA_SIGNATURE_KEY)
            .get(lease)
            .ok()
            .flatten()
            .map(|v| v.wrapped().value);

        // early-out: do we match the latest migration?
        if existing_schema.is_some()
            && self.migrations.last().map(|v| &v.new_signature) == existing_schema.as_ref()
        {
            log::trace!("no migrations to apply");
            return Ok(());
        }

        // early-out: do we have any existing schema at all?
        if existing_schema.is_none() {
            log::trace!("no existing schema found");
            return self.create_from_scratch(lease);
        }

        // we have work to do, time to get started
        let to_apply = self
            .migrations
            .iter()
            .skip_while(|m| Some(&m.old_signature) != existing_schema.as_ref());

        for migration in to_apply {
            log::debug!(
                "Applying migration from {}... to {}...",
                &migration.old_signature[0..8],
                &migration.new_signature[0..8]
            );

            // what entities have *changed* between the two schemas? exclude new/removed
            let table_differences = migration
                .old_schema
                .table_queries()
                .iter()
                .map(|(tname, oq)| {
                    Some((
                        tname,
                        oq,
                        migration.new_schema.table_queries().get(tname.as_str())?,
                    ))
                })
                .flatten()
                .collect::<Vec<_>>();

            let changed = table_differences
                .iter()
                .map(|v| v.0)
                .cloned()
                .collect::<HashSet<_>>();

            let nschema = migration.migration.collect_intermediate_schema(changed);

            // collect_from_database(nschema.as_ref());

            log::trace!("nschema built");

            println!("table_differences: {table_differences:?}");
            todo!()
        }

        log::info!("Database migrations applied");

        Ok(())
    }

    fn create_from_scratch(&self, lease: &mut ConnectionLease) -> DBResult<()> {
        let latest_schema = self
            .migrations
            .last()
            .map(|m| &m.new_schema)
            .unwrap_or(&self.base_schema);

        latest_schema.create(lease)?;

        Ok(())
    }
}
